; Author:      Roy Klein Gebbinck (www.b-k-g.nl)
; Version:     20171219: Revised.
;              20171214: Minor change to ImportSettings.
;              20171213: First version.
; Created for: https://forum.bricsys.com/discussion/32575/user-profile-manager-settings

; ************************
; * USE AT YOUR OWN RISK *
; ************************

; BricsCAD V17 or higher is required (new getcfg and setcfg):
(if (> 17 (atoi (getvar '_vernum)))
  (progn
    (princ "\nImportSettings requires BricsCAD V17 or higher ")
    (exit)
  )
)

; 20171219: Revised.
; 20171219: Added the skipValStr argument.
; 20171219: Removed 'not saved' check.
; 20171214: Added 'value is already set' check.
; 20171213
; Fields in settings file:
; 0    1         2         3       4             5             6      7
; Name,Save mode,Save type,Restype,Default value,Current value,Status,Title
;
; Arguments:
;   Fnm:        File name or nil. If a file name is provided the alert message is suppressed.
;   Mode:       "DEFAULT" or "CURRENT".
;   SkipValStr: Values to skip as string or nil, wild-cards allowed, case-insensitve.
; Examples:
;   (ImportSettings nil "CURRENT" nil)
;   (ImportSettings "D:\\Test\\settings.csv" "CURRENT" "*/*,*\\*")
;   (ImportSettings "D:\\Test\\settings.csv" "CURRENT" "*V18*")
(defun ImportSettings (fnm mode skipValStr / noAlertP failLst fnm idx lst nme skipValLst val)
  (setq noAlertP (and fnm))
  (if
    (and
      (or
        fnm
        (setq fnm (getfiled "Select Settings File" "" "csv" 4))
      )
      (setq lst (cddr (vle-file->list fnm nil))) ; Remove headers and empty line.
      (setq lst (mapcar '(lambda (str) (vle-string-split "," str)) lst))
      (or
        noAlertP
        (=
          1
          (vle-alert
            "ImportSettings"
            (strcat
              "\nThis program will try to import non-dwg related settings from:"
              "\n" fnm
              "\n"
              "\nAre you sure you want to continue?"
            )
            305
          )
        )
      )
    )
    (progn
      (setq idx (if (= "DEFAULT" (strcase mode)) 4 5))
      (setq skipValStr
        (cond
          ((not skipValStr)
            nil
          )
          ((= "" skipValStr)
            nil
          )
          (T
            (strcase skipValStr)
          )
        )
      )
      (foreach sub lst
        (if
          (and
            (getcfg (setq nme (vle-nth0 sub))) ; Check if the setting is available.
            (/= (setq val (nth idx sub)) "")   ; Skip settings that do not have a (default) value.
            (/= (vle-nth1 sub) "drw")          ; Skip settings that are drawing-related.
            (/= (vle-nth6 sub) "read only")    ; Skip read-only settings.
          )
          (cond
            ((and skipValStr (wcmatch (strcase val) skipValStr))
              (setq skipValLst (cons val skipValLst))
            )
            ((= (getcfg nme) val) ; Value is already set.
              nil
            )
            ((not (setcfg nme val))
              (setq failLst (cons nme failLst))
            )
          )
        )
      )
      (if skipValLst
        (progn
          (princ "\n\n****************************************************** ")
          (princ "\nThe following value(s) were skipped: ")
          (foreach str (reverse skipValLst) (princ (strcat "\n  " str)))
          (princ "\n****************************************************** ")
        )
      )
      (if failLst
        (progn
          (princ "\n\n****************************************************** ")
          (princ "\nThe following setting(s) could not be changed: ")
          (foreach str (reverse failLst) (princ (strcat "\n  " str)))
          (princ "\n****************************************************** ")
        )
      )
      (princ "\n\nDone ")
    )
  )
  (princ)
)

; 20171219: The arguments of ImportSettings have changed.
; 20171213
(defun c:ImportSettingsDefaultValues ()
  (ImportSettings nil "DEFAULT" nil)
)

; 20171219: The arguments of ImportSettings have changed.
; 20171213
(defun c:ImportSettingsUserValues ()
  (ImportSettings nil "CURRENT" nil)
)

; 20171219
(defun c:ImportSettingsUserValues_SkipPaths ()
  (ImportSettings nil "CURRENT" "*/*,*\\*")
)

(princ "\nUse ImportSettingsUserValues, ImportSettingsDefaultValues or ImportSettingsUserValues_SkipPaths ")
(princ)
